unit BookmarkModule;

{Ȃ肢Ȃ}

interface

uses
	Windows, SysUtils, ActiveX, Types,
	{$IFNDEF EXTENSION}LObjects, {$ENDIF} EOFileIntf,
	FavoriteModule, RecentModule, ToXXXModule;

type
	TToBookmarkActionItem = class(TInterfacedObject, IActionItem)
	private
		FDir: WideString;
		FChecked: Boolean;
		function GetText: WideString;
		function GetChecked: Boolean;
		function GetItems: IActionItems;
		function Execute: Boolean;
	public
		constructor Create(const ADir: WideString; AChecked: Boolean);
	end;

	TToBookmarkActionItems = class(TInterfacedObject, IActionItems)
	private
		FItems: array of IActionItem;
		FNeedUpdate: Boolean;
		{IActionItems}
		function GetTitle: WideString;
		function IActionItems.Enumerate = EnumerateItems;
		function EnumerateItems(Proc: CEachActionItem; Context: Pointer): IActionItem;
		procedure PostUpdate;
	public
		constructor Create;
		procedure Update;
	end;

	TToBookmarkAction = class(TInterfacedObject, IAction)
	private
		FItems: IActionItems;
		{IActionItem}
		function GetText: WideString;
		function GetChecked: Boolean;
		function GetItems: IActionItems;
		function Execute: Boolean;
		{IAction}
		function GetName: string;
		function GetHint: WideString;
		function GetEnabled: Boolean;
	public
		procedure AfterConstruction; override;
		procedure PostUpdate;
	end;

	TBookmarkAction = class(TInterfacedObject, IAction, IObserver)
	private
		FToBookmark: TToBookmarkAction;
		FEnabled: Boolean;
		{IActionItem}
		function GetText: WideString;
		function GetChecked: Boolean;
		function GetItems: IActionItems;
		function Execute: Boolean;
		{IAction}
		function GetName: string;
		function GetHint: WideString;
		function GetEnabled: Boolean;
		{IObserver}
		procedure Observe;
	public
		constructor Create(AToBookmark: TToBookmarkAction);
	end;

const
	ToBookmarkName = 'ToBookmark';
	BookmarkName = 'Bookmark';

type
	TGlobalExtension = class(TInterfacedObject, IExtension, IExtensionInfo, IThreadHandler)
	private
		{IExtension}
		procedure Initialize;
		procedure Finalize;
		{IExtensionInfo}
		function GetName: WideString;
		procedure Dialog;
		{IThreadHandler}
		procedure ThreadBegun(const AThread: IInterface);
		procedure ThreadEnded(const AThread: IInterface);
	end;

var
	Bookmark: TWideStringDynArray;
	BookmarkModified: Boolean;

implementation

function BookmarkFileName: WideString;
begin
	Result := (GetApplication as IEOFileGlobal).ExePath + 'bookmark.txt';
end;

procedure LoadBookmark;
var
	F: Text;
	FileName, Line: string;
	R: Boolean;
begin
	FileName := BookmarkFileName;
	try
		Assign(F, FileName);
		Reset(F);
		R := False;
		while not Eof(F) do
		begin
			ReadLn(F, Line);
			if Line = '' then
				R := True
			else if R then
			begin
				SetLength(Recent, Length(Recent) + 1);
				Recent[High(Recent)] := UTF8Decode(Line);
			end
			else
			begin
				SetLength(Bookmark, Length(Bookmark) + 1);
				Bookmark[High(Bookmark)] := UTF8Decode(Line);
			end
		end;
		Close(F);
	except
	on EInOutError do ;
	end;
end;

procedure SaveBookmark;
var
	I: Integer;
	F: Text;
	FileName: string;
begin
	FileName := BookmarkFileName;
	try
		Assign(F, FileName);
		Rewrite(F);
		for I := Low(Bookmark) to High(Bookmark) do WriteLn(F, UTF8Encode(Bookmark[I]));
		WriteLn(F);
		for I := Low(Recent) to High(Recent) do WriteLn(F, UTF8Encode(Recent[I]));
		Close(F);
	except
	on EInOutError do ;
	end;
end;

{ TBookmarkAction }

constructor TBookmarkAction.Create(AToBookmark: TToBookmarkAction);
begin
	FToBookmark := AToBookmark;
	inherited Create;
end;

function TBookmarkAction.Execute: Boolean;
var
	Folder: IFolder;
	Sync: IReadWriteSyncEx;
begin
	Result := FEnabled;
	if Result then
	begin
		Sync := GetApplication as IReadWriteSyncEx;
		Sync.BeginWrite;
		try
			Folder := (GetCurrentThread as IEOFileThread).ActiveView.Model;

			AddHistory(Bookmark,
				(GetApplication as IFolderManager).ToString(Folder.Path), 20);

			BookmarkModified := True;
			FToBookmark.PostUpdate;
		finally
			Sync.EndWrite
		end
	end;
end;

function TBookmarkAction.GetChecked: Boolean;
begin
	Result := False
end;

function TBookmarkAction.GetEnabled: Boolean;
begin
	Result := FEnabled
end;

function TBookmarkAction.GetHint: WideString;
begin
	Result := '݈ʒuubN}[N'
end;

function TBookmarkAction.GetItems: IActionItems;
begin
	Result := nil
end;

function TBookmarkAction.GetName: string;
begin
	Result := BookmarkName
end;

function TBookmarkAction.GetText: WideString;
begin
	Result := 'ubN}[N'
end;

procedure TBookmarkAction.Observe;
var
	Model: IFolder;
begin
	Model := (GetCurrentThread as IEOFileThread).ActiveView.Model;
	FEnabled :=
		Supports(Model, IDirectory) or Supports(Model, INameSpace) or
		Supports(Model, IArchive);
end;

{ TToBookmarkAction }

procedure TToBookmarkAction.AfterConstruction;
begin
	inherited;
	FItems := TToBookmarkActionItems.Create
end;

function TToBookmarkAction.Execute: Boolean;
var
	Selected: IActionItem;
begin
	Result := GetEnabled;
	if Result then
	begin
		Selected := (GetApplication as IActionManager).Select(GetItems);
		if Selected <> nil then Selected.Execute
	end
end;

function TToBookmarkAction.GetChecked: Boolean;
begin
	Result := False
end;

function TToBookmarkAction.GetEnabled: Boolean;
begin
	Result := Bookmark <> nil;
end;

function TToBookmarkAction.GetHint: WideString;
begin
	Result := 'ubN}[Nֈړ'
end;

function TToBookmarkAction.GetItems: IActionItems;
begin
	Result := FItems;
end;

function TToBookmarkAction.GetName: string;
begin
	Result := ToBookmarkName
end;

function TToBookmarkAction.GetText: WideString;
begin
	Result := 'ubN}[Nֈړ'
end;

procedure TToBookmarkAction.PostUpdate;
begin
	FItems.PostUpdate
end;

{ TToBookmarkActionItem }

constructor TToBookmarkActionItem.Create(const ADir: WideString; AChecked: Boolean);
begin
	inherited Create;
	FDir := ADir;
	FChecked := AChecked
end;

function TToBookmarkActionItem.Execute: Boolean;
var
	Path: TPath;
	Application: IUnknown;
	FolderManager: IFolderManager;
begin
	Application := GetApplication;
	FolderManager := Application as IFolderManager;

	Path := FolderManager.ToPath(FDir);

	(Application as IReadWriteSyncEx).BeginWrite;
	try
		(GetCurrentThread as IEOFileThread).ActiveView.Model := FolderManager.GetFolder(Path);
	finally
		(Application as IReadWriteSyncEx).EndWrite;
		if Path.ItemIDList <> nil then CoTaskMemFree(Path.ItemIDList);
	end;
	Result := True
end;

function TToBookmarkActionItem.GetChecked: Boolean;
begin
	Result := FChecked
end;

function TToBookmarkActionItem.GetItems: IActionItems;
begin
	Result := nil
end;

function TToBookmarkActionItem.GetText: WideString;
begin
	Result := FDir
end;

{ TToBookmarkActionItems }

constructor TToBookmarkActionItems.Create;
begin
	inherited Create;
	FNeedUpdate := True;
end;

function TToBookmarkActionItems.EnumerateItems(Proc: CEachActionItem; Context: Pointer): IActionItem;
var
	I: Integer;
begin
	if FNeedUpdate then
	begin
		FNeedUpdate := False;
		Update;
	end;
	for I := Low(FItems) to High(FItems) do
	begin
		Result := FItems[I];
		if Proc(Result, Context) then Exit;
	end;
	Result := nil
end;

function TToBookmarkActionItems.GetTitle: WideString;
begin
	Result := 'ubN}[Nֈړ'
end;

procedure TToBookmarkActionItems.PostUpdate;
begin
  FNeedUpdate := True
end;

procedure TToBookmarkActionItems.Update;
var
	I: Integer;
	Cur: WideString;
begin
	Cur := (GetCurrentThread as IEOFileThread).ActiveView.Model.Path.Directory;

	FItems := nil;

	SetLength(FItems, Length(Bookmark));

	for I := High(Bookmark) downto Low(Bookmark) do
	begin
		FItems[I] := TToBookmarkActionItem.Create(
						Bookmark[I],
						WideCompareText(Bookmark[I], Cur) = 0);
	end;
end;

{ TGlobalExtension }

procedure TGlobalExtension.Dialog;
const
	S = 'written by YT.';
begin
	MessageBoxW(0, S, 'Bookmark & Favorite & Recent', MB_ICONINFORMATION)
end;

procedure TGlobalExtension.Finalize;
begin
end;

function TGlobalExtension.GetName: WideString;
begin
	Result := 'Bookmark && Favorite'
end;

procedure TGlobalExtension.Initialize;
begin
end;

procedure TGlobalExtension.ThreadBegun(const AThread: IInterface);
var
	ToBookmark: TToBookmarkAction;
	Bookmark: TBookmarkAction;
	Favorite: TFavoriteAction;
	Recent: TRecentAction;
	ToRecycleBin: TToRecycleBinAction;
begin
	ToBookmark := TToBookmarkAction.Create;
	(AThread as IActionManager).Attach(ToBookmark);

	Bookmark := TBookmarkAction.Create(ToBookmark);
	(AThread as IObservable).Attach(Bookmark);
	(AThread as IActionManager).Attach(Bookmark);

	Favorite := TFavoriteAction.Create;
	(AThread as IObservable).Attach(Favorite);
	(AThread as IActionManager).Attach(Favorite);

	Recent := TRecentAction.Create;
	(AThread as IObservable).Attach(Recent);
	(AThread as IActionManager).Attach(Recent);

	ToRecycleBin := TToRecycleBinAction.Create;
	(AThread as IObservable).Attach(ToRecycleBin);
	(AThread as IActionManager).Attach(ToRecycleBin);
end;

procedure TGlobalExtension.ThreadEnded(const AThread: IInterface);
begin
end;

procedure Register;
begin
	with GetApplication as IExtensionManager do
	begin
		RegisterExtension(TGlobalExtension.Create);
		RegisterExtension(TRecentExtension.Create);
	end
end;

var
	Installed: Boolean;

initialization

	Installed := GetInterfaceVersion = EOFileIntf.InterfaceVersion;
	if Installed then
	begin
		LoadBookmark;
		Register;
	end

finalization

	if Installed then
	begin
		if BookmarkModified or RecentModified then SaveBookmark;
	end

end.
