unit RecentModule;

interface

uses
	ActiveX, SysUtils, Types,
	{$IFNDEF EXTENSION}LObjects, {$ENDIF} EOFileIntf;

type
	TRecentActionItem = class(TInterfacedObject, IActionItem)
	private
		FDir: WideString;
		FChecked: Boolean;
		function GetText: WideString;
		function GetChecked: Boolean;
		function GetItems: IActionItems;
		function Execute: Boolean;
	public
		constructor Create(const ADir: WideString; AChecked: Boolean);
	end;

	TRecentActionItems = class(TInterfacedObject, IActionItems)
	private
		FItems: array of IActionItem;
		FNeedUpdate: Boolean;
		function GetTitle: WideString;
		function Enumerate(Proc: CEachActionItem; Context: Pointer): IActionItem;
		procedure PostUpdate;
	public
		constructor Create;
		procedure Update;
	end;

	TRecentAction = class(TInterfacedObject, IAction, IObserver)
	private
		FItems: IActionItems;
		{IActionItem}
		function GetText: WideString;
		function GetChecked: Boolean;
		function GetItems: IActionItems;
		function Execute: Boolean;
		{IAction}
		function GetName: string;
		function GetHint: WideString;
		function GetEnabled: Boolean;
		{IObserver}
		procedure Observe;
	public
    procedure AfterConstruction; override;
	end;

	TRecentExtension = class(TInterfacedObject, IExtension, IViewHandler)
	private
		{IExtension}
		procedure Finalize;
		procedure Initialize;
		{IViewHandler}
		procedure OnOpen(const View: IView);
		procedure OnClose(const View: IView);
	end;

const
	RecentName = 'Recent';

var
	Recent: TWideStringDynArray;
	RecentModified: Boolean;

procedure AddHistory(var A: TWideStringDynArray; const T: WideString; Limit: Integer);

implementation

procedure AddHistory(var A: TWideStringDynArray; const T: WideString; Limit: Integer);
var
	F, I: Integer;
begin
	F := Length(A);
	for I := High(A) downto Low(A) do
		if A[I] = T then
		begin
			F := I;
			Break;
		end;
	if F = Length(A) then SetLength(A, F + 1);
	for I := F downto 1 do A[I] := A[I - 1];
	A[0] := T;
	if Length(A) > Limit then SetLength(A, Limit);
end;

{ TRecentActionItem }

constructor TRecentActionItem.Create(const ADir: WideString; AChecked: Boolean);
begin
	inherited Create;
	FDir := ADir;
	FChecked := AChecked
end;

function TRecentActionItem.Execute: Boolean;
var
	Path: TPath;
	Application: IUnknown;
	FolderManager: IFolderManager;
	Syn: IReadWriteSyncEx;
begin
	Application := GetApplication;
	Syn := Application as IReadWriteSyncEx;
	FolderManager := Application as IFolderManager;

	Path := FolderManager.ToPath(FDir);

	Syn.BeginWrite;
	try
		(GetCurrentThread as IEOFileThread).ActiveView.Model := FolderManager.GetFolder(Path);
	finally
		Syn.EndWrite;
		if Path.ItemIDList <> nil then CoTaskMemFree(Path.ItemIDList);
	end;
	Result := True
end;

function TRecentActionItem.GetChecked: Boolean;
begin
	Result := FChecked
end;

function TRecentActionItem.GetItems: IActionItems;
begin
	Result := nil
end;

function TRecentActionItem.GetText: WideString;
begin
	Result := FDir
end;

{ TRecentActionItems }

constructor TRecentActionItems.Create;
begin
	inherited Create;
  FNeedUpdate := True
end;

function TRecentActionItems.Enumerate(Proc: CEachActionItem; Context: Pointer): IActionItem;
var
	I: Integer;
begin
	if FNeedUpdate then
	begin
		FNeedUpdate := False;
		Update;
	end;
	for I := Low(FItems) to High(FItems) do
	begin
		Result := FItems[I];
		if Proc(Result, Context) then Exit;
	end;
	Result := nil
end;

function TRecentActionItems.GetTitle: WideString;
begin
	Result := 'ŋߕtH_'
end;

procedure TRecentActionItems.PostUpdate;
begin
	FNeedUpdate := True
end;

procedure TRecentActionItems.Update;
var
	I: Integer;
	Cur: WideString;
begin
	Cur := (GetCurrentThread as IEOFileThread).ActiveView.Model.Path.Directory;

	FItems := nil;

	SetLength(FItems, Length(Recent));

	for I := High(Recent) downto Low(Recent) do
	begin
		FItems[I] := TRecentActionItem.Create(Recent[I],
			WideCompareText(Recent[I], Cur) = 0);
	end;
end;

{ TRecentAction }

procedure TRecentAction.AfterConstruction;
begin
  inherited;
	FItems := TRecentActionItems.Create
end;

function TRecentAction.Execute: Boolean;
var
	Selected: IActionItem;
begin
	Result := GetEnabled;
	if Result then
	begin
		Selected := (GetApplication as IActionManager).Select(FItems);
		if Selected <> nil then Selected.Execute
	end
end;

function TRecentAction.GetChecked: Boolean;
begin
	Result := False
end;

function TRecentAction.GetEnabled: Boolean;
begin
	Result := Recent <> nil;
end;

function TRecentAction.GetHint: WideString;
begin
	Result := 'ŋߕtH_ֈړ܂'
end;

function TRecentAction.GetItems: IActionItems;
begin
	Result := FItems
end;

function TRecentAction.GetName: string;
begin
	Result := RecentName
end;

function TRecentAction.GetText: WideString;
begin
	Result := 'ŋߕtH_'
end;

procedure TRecentAction.Observe;
begin
	FItems.PostUpdate
end;

{ TRecentExtension }

procedure TRecentExtension.Finalize;
begin
end;

procedure TRecentExtension.Initialize;
begin
end;

procedure TRecentExtension.OnClose(const View: IView);
var
	App: IUnknown;
	Syn: IReadWriteSyncEx;
begin
	App := GetApplication;
	Syn := App as IReadWriteSyncEx;
	Syn.BeginWrite;
	try
		AddHistory(Recent, (App as IFolderManager).ToString(View.Model.Path), 20);
		RecentModified := True;
	finally
		Syn.EndWrite
	end;
end;

procedure TRecentExtension.OnOpen(const View: IView);
begin
end;

end.
